import React, { useEffect, useState } from 'react';
import { DatabaseType, SaveStateI, databaseNameMapping, databaseProtocolMapping, nilUUID } from '@graphpolaris/shared/lib/data-access';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { useImmer } from 'use-immer';
import { initialState as qbInitialState } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';

export const INITIAL_SAVE_STATE: SaveStateI = {
  id: nilUUID,
  name: 'Untitled',
  db: {
    username: 'neo4j',
    password: 'DevOnlyPass',
    url: 'localhost',
    port: 7687,
    protocol: 'neo4j://',
    internalDatabaseName: 'neo4j',
    type: DatabaseType.Neo4j,
  },
  schema: {},
  queryBuilder: qbInitialState,
  visualization: {},
  share_state: {},
  user_id: '',
};

export const DatabaseForm = (props: { data: SaveStateI; onChange: (data: SaveStateI, error: boolean) => void }) => {
  const [formData, setFormData] = useImmer(props.data);
  const [hasError, setHasError] = useState<Record<string, boolean>>({});

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port)))
      setFormData((draft) => {
        draft.db.port = Number(port);
        return draft;
      });
  }

  useEffect(() => {
    props.onChange(formData, Object.values(hasError).includes(true));
  }, [formData]);

  return (
    <div className="flex flex-col gap-4 my-2">
      <Input
        type="text"
        label="Name of database connection"
        value={formData.name}
        onChange={(value: string) =>
          setFormData((draft) => {
            draft.name = value;
          })
        }
      />

      <Input
        type="text"
        label="Internal database name"
        value={formData.db.internalDatabaseName}
        placeholder="internalDatabaseName"
        required
        errorText="This field is required"
        validate={(v) => {
          setHasError({ ...hasError, internalDatabaseName: v.length === 0 });
          return v.length > 0;
        }}
        onChange={(value: string) =>
          setFormData((draft) => {
            draft.db.internalDatabaseName = value;
          })
        }
      />

      <div className="flex w-full gap-2">
        <Input
          type="dropdown"
          className="w-full"
          label="Database Type"
          required
          value={databaseNameMapping[formData.db.type]}
          options={databaseNameMapping}
          onChange={(value: string | number) => {
            setFormData((draft) => {
              draft.db.type = databaseNameMapping.indexOf(value.toString());
            });
          }}
        />

        <Input
          type="dropdown"
          label="Database Protocol"
          required
          value={formData.db.protocol}
          options={databaseProtocolMapping}
          info="Protocol via which the database connection will be established"
          onChange={(value: string | number) => {
            setFormData((draft) => {
              draft.db.protocol = value.toString();
            });
          }}
        />
      </div>

      <div className="flex w-full gap-2">
        <Input
          type="text"
          label="Hostname/IP"
          value={formData.db.url}
          placeholder="neo4j"
          required
          errorText="This field is required"
          info="Connection URL of your database"
          validate={(v) => {
            setHasError({ ...hasError, url: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value: string) => {
            setFormData((draft) => {
              draft.db.url = value;
            });
          }}
        />

        <Input
          type="text"
          label="Port"
          value={formData.db.port.toString()}
          placeholder="neo4j"
          required
          errorText="Must be between 1 and 9999"
          info="The port is endpoint of service for communication purposes"
          validate={(v) => {
            setHasError({ ...hasError, port: !(v <= 9999 && v > 0) });
            return v <= 9999 && v > 0;
          }}
          onChange={(value: string) => {
            setFormData((draft) => {
              draft.db.port = Number(value);
            });
          }}
        />
      </div>

      <div className="flex w-full gap-2">
        <Input
          type="text"
          label="Username"
          value={formData.db.username}
          placeholder="username"
          required
          errorText="This field is required"
          info="Username of your database instance"
          validate={(v) => {
            setHasError({ ...hasError, username: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value: string) => {
            setFormData((draft) => {
              draft.db.username = value;
            });
          }}
        />

        <Input
          type="text"
          visible={false}
          label="Password"
          value={formData.db.password}
          placeholder="password"
          required
          errorText="This field is required"
          info="Password of your database instance"
          validate={(v) => {
            setHasError({ ...hasError, password: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value: string) => {
            setFormData((draft) => {
              draft.db.password = value;
            });
          }}
        />
      </div>
    </div>
  );
};
